# To use SymEngine from another CMake project include the following in your
# `CMakeLists.txt` file

#    `find_package(SymEngine CONFIG)`

# You can give the path to the SymEngine installation directory if it was
# installed to a non standard location by,

#    `find_package(SymEngine CONFIG Paths /path/to/install/dir)`

# Alternatively, you can give the path to the build directory.

# Variable exported are
# SYMENGINE_BUILD_TYPE         - Cofiguration Type Debug or Release
# SYMENGINE_INCLUDE_DIRS       - Header file directories
# SYMENGINE_LIBRARIES          - SymEngine libraries and dependency libraries to link against
# SYMENGINE_FOUND              - Set to yes
# SYMENGINE_CXX_FLAGS_RELEASE  - C++ flags for Release configuration
# SYMENGINE_CXX_FLAGS_DEBUG    - C++ flags for Debug configuration
# SYMENGINE_C_FLAGS_RELEASE    - C flags for Release configuration
# SYMENGINE_C_FLAGS_DEBUG      - C flags for Debug configuration

# An example project would be,
#
# cmake_minimum_required(VERSION 2.8)
# find_package(symengine CONFIG)
# set(CMAKE_CXX_FLAGS_RELEASE ${SYMENGINE_CXX_FLAGS_RELEASE})
#
# include_directories(${SYMENGINE_INCLUDE_DIRS})
# add_executable(example main.cpp)
# target_link_libraries(example ${SYMENGINE_LIBRARIES})
#

cmake_minimum_required(VERSION 2.8)

set(SYMENGINE_CXX_FLAGS_RELEASE "-std=c++11 -Wall -Wextra -fno-common -fPIC -O3 -ffast-math -funroll-loops -Wno-unused-parameter -march=native -D_GNU_SOURCE -D__STDC_CONSTANT_MACROS -D__STDC_FORMAT_MACROS -D__STDC_LIMIT_MACROS -DNDEBUG -fno-builtin-malloc -fno-builtin-calloc -fno-builtin-realloc -fno-builtin-free -fopenmp")
set(SYMENGINE_CXX_FLAGS_DEBUG "-std=c++11 -Wall -Wextra -fno-common -fPIC -g -Wno-unused-parameter -ggdb -D_GNU_SOURCE -D__STDC_CONSTANT_MACROS -D__STDC_FORMAT_MACROS -D__STDC_LIMIT_MACROS -fno-builtin-malloc -fno-builtin-calloc -fno-builtin-realloc -fno-builtin-free -fopenmp")
set(SYMENGINE_C_FLAGS_RELEASE "-O3 -DNDEBUG")
set(SYMENGINE_C_FLAGS_DEBUG "-g")

# ... for the build tree
get_filename_component(SYMENGINE_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

set(SYMENGINE_BUILD_TREE no)

if(NOT TARGET symengine)
    include("${SYMENGINE_CMAKE_DIR}/SymEngineTargets.cmake")
endif()

if(SYMENGINE_BUILD_TREE)
    set(CMAKE_MODULE_PATH "${SYMENGINE_CMAKE_DIR}/cmake/")
    set(SYMENGINE_INSTALL_CMAKE_DIR "${SYMENGINE_CMAKE_DIR}")
    set(SYMENGINE_INCLUDE_DIRS /home/abuild/rpmbuild/BUILD/symengine-0.3.0;/home/abuild/rpmbuild/BUILD/symengine-0.3.0/build;/home/abuild/rpmbuild/BUILD/symengine-0.3.0/symengine/utilities/teuchos;/home/abuild/rpmbuild/BUILD/symengine-0.3.0/build/symengine/utilities/teuchos ${SYMENGINE_CMAKE_DIR})
    if (TARGET teuchos)
        set(SYMENGINE_INCLUDE_DIRS ${SYMENGINE_INCLUDE_DIRS} ${SYMENGINE_CMAKE_DIR}/symengine/teuchos)
    endif()
else()
    set(CMAKE_MODULE_PATH ${SYMENGINE_CMAKE_DIR})
    set(SYMENGINE_INSTALL_CMAKE_DIR "/usr//usr/lib64/cmake/symengine")
    set(SYMENGINE_INCLUDE_DIRS "${SYMENGINE_CMAKE_DIR}/../../../include")
endif()


set(SYMENGINE_GMP_LIBRARIES /usr/lib64/libgmp.so)
set(SYMENGINE_GMP_INCLUDE_DIRS /usr/include)
set(SYMENGINE_ECM_LIBRARIES /usr/lib64/libecm.so)
set(SYMENGINE_ECM_INCLUDE_DIRS /usr/include)
set(SYMENGINE_MPC_LIBRARIES /usr/lib64/libmpc.so)
set(SYMENGINE_MPC_INCLUDE_DIRS /usr/include)
set(SYMENGINE_PTHREAD_LIBRARIES /usr/lib64/libpthread.so)
set(SYMENGINE_PTHREAD_INCLUDE_DIRS /usr/include)
set(SYMENGINE_MPFR_LIBRARIES /usr/lib64/libmpfr.so)
set(SYMENGINE_MPFR_INCLUDE_DIRS /usr/include)
set(SYMENGINE_LLVM_LIBRARIES )
set(SYMENGINE_LLVM_INCLUDE_DIRS /usr/include)
set(SYMENGINE_BFD_LIBRARIES /usr/lib64/libbfd.a;/usr/lib64/libiberty.a;/usr/lib64/libz.so;/usr/lib64/libdl.so)
set(SYMENGINE_BFD_INCLUDE_DIRS /usr/include)
set(SYMENGINE_LINKH_LIBRARIES )
set(SYMENGINE_LINKH_INCLUDE_DIRS /usr/include)
set(SYMENGINE_EXECINFO_LIBRARIES )
set(SYMENGINE_EXECINFO_INCLUDE_DIRS /usr/include)
set(SYMENGINE_TCMALLOC_LIBRARIES /usr/lib64/libtcmalloc.so)
set(SYMENGINE_TCMALLOC_INCLUDE_DIRS )

set(SYMENGINE_LLVM_COMPONENTS asmparser;core;executionengine;instcombine;mcjit;native;nativecodegen;scalaropts;support;transformutils)

if (SYMENGINE_LLVM_COMPONENTS)
    find_package(LLVM REQUIRED ${SYMENGINE_LLVM_COMPONENTS})
    llvm_map_components_to_libnames(llvm_libs ${SYMENGINE_LLVM_COMPONENTS})
    set(SYMENGINE_LIBRARIES ${SYMENGINE_LIBRARIES} ${llvm_libs})
endif()

if (TARGET gmp)
    # Avoid defining targets again
    set(SYMENGINE_SKIP_DEPENDENCIES yes CACHE BOOL "Skip finding dependencies")
else()
    set(SYMENGINE_SKIP_DEPENDENCIES no CACHE BOOL "Skip finding dependencies")
endif()

foreach(PKG GMP;ECM;MPC;PTHREAD;MPFR;LLVM;BFD;LINKH;EXECINFO;TCMALLOC)
    set(SYMENGINE_INCLUDE_DIRS ${SYMENGINE_INCLUDE_DIRS} ${SYMENGINE_${PKG}_INCLUDE_DIRS})
    set(SYMENGINE_LIBRARIES ${SYMENGINE_LIBRARIES} ${SYMENGINE_${PKG}_LIBRARIES})
endforeach()

#Use CMake provided find_package(BOOST) module
if (NOT "/usr/include" STREQUAL "")
    set(SYMENGINE_INCLUDE_DIRS ${SYMENGINE_INCLUDE_DIRS} /usr/include)
    set(SYMENGINE_LIBRARIES ${SYMENGINE_LIBRARIES} )
endif()

list(REMOVE_DUPLICATES SYMENGINE_INCLUDE_DIRS)

foreach(LIB "symengine;teuchos")
    # Remove linking of dependencies to later add them as targets
    set_target_properties(${LIB} PROPERTIES IMPORTED_LINK_INTERFACE_LIBRARIES_RELEASE "")
    set_target_properties(${LIB} PROPERTIES IMPORTED_LINK_INTERFACE_LIBRARIES_DEBUG "")
endforeach()

set(SYMENGINE_LIBRARIES symengine;teuchos ${SYMENGINE_LIBRARIES})
set(SYMENGINE_BUILD_TYPE "Release")
set(SYMENGINE_FOUND yes)
